/**
 * Created by senthil on 28/07/17.
 */
'use strict';

const EventEmitter = require('events').EventEmitter;
const path = require('path');
const _ = require('lodash');
const EngineRunner = require('./runner');
var uuid = require('uuid');
var async = require('async');
var pa11y = require('pa11y');
var http = require('http');
var fs = require('fs');

module.exports = {
    runner: runner
};

let ee = new EventEmitter();

function runner(params, options, callback) {
    var script = params.script;
    var self = this;
    let Runner = new EngineRunner(options, ee);
    //_executor(script);
    // Define some URLs to test, and a concurrency
    var concurrency = 1 // Run two tests at a time

    // Use the async library to create a queue. This accepts a
    // function to handle the URLs, and a concurrency.
    var queue = async.queue(function(arg, done) {
        //console.info('in the queue');
        var scenario = arg.scenario;
        var flow = arg.flow;
        var standard = arg.standard;
        var started = new Date();
        var imagePath = script.config.screenCapturePath + arg.urlId + '-' + standard + '.jpg';

        var pa11yOptions = {}
        pa11yOptions.log = {
            debug: console.log.bind(console),
            error: console.error.bind(console),
            info: console.log.bind(console)
        };

        if(script.config.screenCapturePath) {
            pa11yOptions.screenCapture = imagePath;
        }
        pa11yOptions.htmlcs = path.resolve(__dirname, './HTMLCS.js');

        pa11yOptions.standard = arg.standard;

        if(script.config.page) {
            pa11yOptions.page = script.config.page
        }

        if(script.config.wait) {
            pa11yOptions.wait = script.config.wait;
        }

        if(flow.actions) {
            pa11yOptions.actions = flow.actions;
        }

        if (script.config.phantom) {
            pa11yOptions.phantom = script.config.phantom;
        }

        console.info('pa11yOptions = ', JSON.stringify(pa11yOptions, null, 2));
        var test = pa11y(pa11yOptions);

        // The queue function will be called with each URL. We
        // can then run the pa11y test function on them and call
        // `done` when we're finished to free up the queue
        test.run(arg.targetUrl, function(error, results) {
            if (error) {
                return console.error(error.message);
            }
            console.log(results);
            var data = {};
            data.result = results;
            data.requestUrl = {url: arg.targetUrl};
            data.script = script;
            data.standard = standard;
            data.started = started;
            data.urlId = arg.urlId;
            data.sectionId = arg.sectionId;
            data.imagePath = imagePath;
            data.scenario = scenario;
            data.flow = flow;
            data.status = 'COMPLETED';

            ee.emit('a11ystats', data);
            done();
        });

    }, concurrency);

// Add a function that is triggered when the queue
// drains (it runs out of URLs to process)
    queue.drain = function() {
        console.log('All done!');
        callback(null, {status: {code: 200, message: 'Success'}});
    };

// Lastly, push the URLs we wish to test onto the queue
    script.scenarios.forEach(function(scenario) {
        var sId = uuid.v4();
        scenario.flow.forEach(function (flow) {
            var standards = script.config.standards;
            var urlId = uuid.v4();
            var processItems = function(x){
                if( x < standards.length ) {
                    var standardsCounter = 0;
                    var standard = standards[x];
                    var sectionId = uuid.v4();
                    var targetUrl = script.config.target + flow.url;
                    var data = {};
                    data.requestUrl = {url: targetUrl, headers: flow.headers};
                    data.script = script;
                    data.standard = standard;
                    data.started = new Date();
                    data.urlId = urlId;
                    data.sectionId = sectionId;
                    data.scenario = scenario;
                    data.flow = flow;
                    data.status = 'STARTED';

                    ee.emit('a11ysummarystats', data);
                    queue.push({scenario: scenario, flow: flow, scenarioId: sId, urlId : urlId, sectionId : sectionId, targetUrl : targetUrl, standard: standard}, function(result) {
                        processItems(x+1);
                    });
                }
            };

            processItems(0);
        });
    });


};

function test(script, scenario, flow, standardsCounter, urlId, queue, callback) {
    var standard = script.config.standards[standardsCounter];
    var sectionId = uuid.v4();
    var targetUrl = script.config.target + flow.url;
    var data = {};
    data.requestUrl = {url: targetUrl};
    data.script = script;
    data.standard = standard;
    data.started = new Date();
    data.urlId = urlId;
    data.sectionId = sectionId;
    data.scenario = scenario;
    data.flow = flow;
    data.status = 'STARTED';

    ee.emit('a11ysummarystats', data);
    queue.push({scenario: scenario, flow: flow, urlId : urlId, sectionId : sectionId, targetUrl : targetUrl, standard: standard}, function(val) {
        if ((script.config.standards.length - 1) > standardsCounter) {
            test(script, scenario, flow, standardsCounter, urlId, queue, callback);
        } else {
            callback(val);
        }
    });
}

function _executor (script) {
    var counter = 0;
    var standardsCounter = 0;

    _execute(script, counter, standardsCounter, uuid.v4(), function(err, result) {
        console.info('done');
    });

}

function _execute(config, counter, standardsCounter, urlId) {
    console.info('config = ', config);
    var requestUrl = config.requestUrls[counter];
    var standard = config.standards[standardsCounter];
    var started = new Date();

    console.info("requestUrl = ", urlId);
    if (requestUrl) {
        // Create a test instance with some default options
        var imagePath = config.screenCapturePath + urlId + '-' + standard + '.jpg';
        var test = pa11y({

            screenCapture: imagePath,
            // Log what's happening to the console
            log: {
                debug: console.log.bind(console),
                error: console.error.bind(console),
                info: console.log.bind(console)
            },
            htmlcs: '/Users/senthil/work/projects/perf/work/perfx360/routes/service/exec/HTMLCS.js',
            standard: standard,
            page: config.page
        });

        test.run(requestUrl.url, function (error, result) {
            if (error) {
                return console.error(error.message);
            }
            //console.log(result);
            var data = {};
            data.result = result;
            data.requestUrl = requestUrl;
            data.script = config;
            data.standard = standard;
            data.started = started;
            data.urlId = urlId;
            data.imagePath = imagePath;

            ee.emit('a11ystats', data);

            if ((config.standards.length - 1) > standardsCounter) {
                _execute(config, counter, ++standardsCounter, urlId);
            } else {
                _execute(config, ++counter, 0, uuid.v4());
            }

        });
    }
}

